/*
 *  Define the DMA buffering scheme.  Note that in the current setup for
 *  running the buffers, having more than two buffers may improve the
 *  continuity of the DMA, but will cause more lag in the display.
 *  Also, the number of buffers must be 2 or greater.
 */
#define BUFFERS 2
#define MAX_LEN 2048

#define WINDOW_TOP 100
#define WINDOW_BOTTOM 380
#define WINDOW_LEFT 64
#define WINDOW_RIGHT 575

#define SRX 320
#define SRY 10
#define FLX 320
#define FLY 22
#define FRX 320
#define FRY 34
#define WFX 320
#define WFY 46
#define DGX 320
#define DGY 58
#define RFX 320
#define RFY 70

#define GRAPH_COLOR  1
#define LABEL_COLOR  2
#define BORDER_COLOR 3
#define TEXT_COLOR   4
#define LIGHT_HIGHLIGHT 5
#define DARK_HIGHLIGHT 7

#define UP        0x148
#define DOWN      0x150
#define LEFT      0x14b
#define RIGHT     0x14d
#define CTL_LEFT  0x173
#define CTL_RIGHT 0x174

struct rgb
{
   int red;
   int green;
   int blue;
};

/*
 *  Pointer for writing directly to the screen
 */
#define screen(i) (*(unsigned char far *)MK_FP(0xA000,i))

/*
 *  Functions in SETUPSUB.C
 */
void parse_ini_file(void);
void parse_command(int,char *[]);
void setup_buffers(int);
void compute_window_function(void);
void setup_xscale(void);
void setup_linscales(void);
void setup_logscales(void);
void frequency_scale(void);
void amplitude_scale(void);
void update_header(void);

/*
 *  Functions in PROCINP.C
 */
void setup_vga(void);
void cleanup_vga(void);
int process_input(void);

/*
 *  Functions in FFT.C
 */
void InitializeFFT(int);
void EndFFT(void);
void realfft(int *);

/*
 *  External table (generated in REALFFT.C) which provides the bit-reversed
 *  indices for reading the results of the FFT
 */
extern int *BitReversed;

